@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/game.c,v 11.34 2006/01/13 17:57:00 jcb Exp $
+/* $Header: /home/jcb/newmj/RCS/game.c,v 11.35 2007/05/22 13:38:37 jcb Exp $
  * game.c
  * Routines for manipulating game data structures.
  */
@@ -14,7 +14,7 @@
  *                                                           *
  *************************************************************/
  
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/game.c,v 11.34 2006/01/13 17:57:00 jcb Exp $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/game.c,v 11.35 2007/05/22 13:38:37 jcb Exp $";
 
 #include "game.h"
 #include "sysdep.h"
@@ -130,6 +130,7 @@ static GameOptionEntry default_optiontable[] = {
   { GOEastDoubles, "EastDoubles", GOTBool, 0, 0, { 1 }, "east pays and receives double", 0 },
   { GOLosersSettle, "LosersSettle", GOTBool, 0, 0, { 1 }, "losers pay each other", 0 },
   { GODiscDoubles, "DiscDoubles", GOTBool, 0, 0, { 0 }, "the discarder pays double", 0 },
+  { GOShowOnWashout, "ShowOnWashout", GOTBool, 0, 0, { 0}, "tiles revealed on washout", 0 },
   { GOEnd, "End" , GOTBool, 0, 0, { 1 }, "end of option list", 0 }
 };
 
@@ -1740,11 +1741,13 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	  return -1;
 	}
 	if ( p->num_concealed > 0 ) {
-	  if ( g->state != MahJonging ) {
+	  /* the HandComplete case arises when tiles are revealed
+	     after a washout */
+	  if ( g->state != MahJonging && g->state != HandComplete ) {
 	    g->cmsg_err = "Can't reveal your tiles now!";
 	    return -1;
 	  }
-	  if ( s == g->player ) {
+	  if ( g->state == MahJonging && s == g->player ) {
 	    g->cmsg_err = "Must finish making your sets";
 	    return -1;
 	  }
