@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/game.h,v 11.5 2003/04/04 20:05:59 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/game.h,v 11.7 2004/06/24 20:59:15 jcb Exp $
  * game.h
  * This file contains the game data structure and associated definitions,
  * and prototypes for some functions that work on it.
@@ -95,9 +95,11 @@ typedef enum {
 	      */
 } GameState;
 
-/* This is how we index arrays of the players in the current game */
-typedef enum { east = 0, south = 1, west = 2, north = 3 } seats;
-#define noseat ((seats)-1)
+/* This is how we index arrays of the players in the current game.
+   We used to have  noseat  #defined to be (seats)-1.
+   However, negative enums are allowed, so use them! */
+
+typedef enum { east = 0, south = 1, west = 2, north = 3, noseat = -1 } seats;
 #define NUM_SEATS 4
 
 /* typedef for where the extra tile came from */
@@ -331,6 +333,12 @@ int game_set_option_in_table(GameOptionTable *t, GameOptionEntry *e);
 /* set an option. Function will allocate space as required */
 int game_set_option(Game *g, GameOptionEntry *e);
 
+/* debugging function: return a (possibly static) buffer holding a
+   textual dump of the game state. Second argument returns number
+   of spare bytes in the returned buffer
+*/
+char *game_print_state(Game *g, int *bytes_left);
+
 /* include enum parsing options */
 #include "game-enums.h"
 
