@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/greedy.c,v 11.8 2006/01/14 14:58:14 jcb Exp $
+/* $Header: /home/jcb/newmj/RCS/greedy.c,v 11.13 2006/12/16 17:14:55 jcb Exp $
  * greedy.c
  * This is a computer player. Currently offensive only.
  * Options not documented.
@@ -15,7 +15,7 @@
  *                                                           *
  *************************************************************/
  
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/greedy.c,v 11.8 2006/01/14 14:58:14 jcb Exp $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/greedy.c,v 11.13 2006/12/16 17:14:55 jcb Exp $";
 
 static int debugeval = 0;
 #include <stdlib.h>
@@ -26,6 +26,19 @@ static int debugeval = 0;
 
 #include "version.h"
 
+static double magic[28+13] = 
+  {
+    12, 2, 1, 0.15, 1, 4, 4.97465626805558, 4, 1, 0.00638363133042069,
+    0, 1, 0.025, 0.4, 0.34, 0.4, 0.033333333, 0.3, 
+    2, 25, 2, 25, 90.2519293502827, 3, 0.99, 1, -0.0011160627477312, 0.25,
+
+    6.0, 2.0, 25.0, 1.0, 12.0, 0.5, 3.0, 5.0, 0.5, 0.5, 0.5, 0.5,
+    0.5 };
+
+static double *magic2 = &magic[28];
+
+
+
 /* is this tile a doubling tile (or scoring pair) */
 #define is_doubler(t) (is_dragon(t) || (is_wind(t) && \
  (suit_of(t) == our_player->wind || suit_of(t) == the_game->round)))
@@ -35,6 +48,8 @@ int our_id = 0;
 PlayerP our_player;
 seats our_seat;
 
+static FILE *debugf;
+
 /* New style strategy */
 typedef struct {
   double chowness; /* how much do we like/dislike chows? 
@@ -128,6 +143,8 @@ int main(int argc, char *argv[]) {
   char *address = ":5000";
   char *name = NULL;
 
+  debugf = stderr;
+
   /* options. I should use getopt ... */
   for (i=1;i<argc;i++) {
     if ( strcmp(argv[i],"--id") == 0 ) {
@@ -164,8 +181,37 @@ int main(int argc, char *argv[]) {
     } else if ( strcmp(argv[i],"--suitness") == 0 ) {
       if ( ++i == argc ) usage(argv[0],"missing argument to --suitness");
       parsefloatlist(argv[i],suitness);
-    }
-    else {
+    } else if ( strcmp(argv[i],"--magic") == 0 ) {
+      FILE *f;
+      int j;
+      if ( ++i == argc ) usage(argv[0],"missing argument to --magic");
+      f = fopen(argv[i],"r");
+      if ( f == NULL ) {
+	perror("can't open param file");
+	exit(1);
+      }
+      for ( j = 0 ; j < sizeof(magic)/sizeof(double) ; j++ ) {
+	if ( fscanf(f,"%lg",&magic[j]) != 1 ) {
+	  fprintf(stderr,"%s","error in param file\n");
+	  exit(1);
+	}
+      }
+    } else if ( strcmp(argv[i],"--magic2") == 0 ) {
+      FILE *f;
+      int j;
+      if ( ++i == argc ) usage(argv[0],"missing argument to --magic2");
+      f = fopen(argv[i],"r");
+      if ( f == NULL ) {
+	perror("can't open param file");
+	exit(1);
+      }
+      for ( j = 0 ; j < 13 ; j++ ) {
+	if ( fscanf(f,"%lg",&magic2[j]) != 1 ) {
+	  fprintf(stderr,"%s","error in param file\n");
+	  exit(1);
+	}
+      }
+    } else {
       fprintf(stderr,argv[i]);
       usage(argv[0],"unknown option or argument");
     }
@@ -256,7 +302,7 @@ static int despatch_line(char *line) {
   case CMsgNewHand:
     game_handle_cmsg(the_game,cm);
     our_seat = game_id_to_seat(the_game,our_id);
-    if ( debugeval ) printf("New hand\n");
+    if ( debugeval ) fprintf(debugf,"New hand\n");
     /* reset strategy to default */
     curstrat.chowness = stratparams[chowness].values[0];
     curstrat.hiddenness = stratparams[hiddenness].values[0];
@@ -608,11 +654,11 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
 
   if ( debugeval ) {
     player_print_tiles(buf,p,0);
-    printf("Hand: %s  ; discard %s\n",buf,tile_code(the_game->tile));
+    fprintf(debugf,"Hand: %s  ; discard %s\n",buf,tile_code(the_game->tile));
   }
   bestval = evalhand(p,strat);
   if ( debugeval ) {
-    printf("Hand value before claim %.3f\n",bestval);
+    fprintf(debugf,"Hand value before claim %.3f\n",bestval);
   }
 
   canmj = player_can_mah_jong(p,the_game->tile,mjspecflags);
@@ -626,7 +672,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
       val = evalhand(&pc,strat);
       /* we won't discard a tile here */
       if ( debugeval ) {
-	printf("Hand after kong  %.3f\n",val);
+	fprintf(debugf,"Hand after kong  %.3f\n",val);
       }
       if ( val > bestval ) {
 	m.type = PMsgKong;
@@ -635,7 +681,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
       }
       else 
 	if ( debugeval ) {
-	  printf("Chose not to kong\n");
+	  fprintf(debugf,"Chose not to kong\n");
 	}
   } 
   if ( player_can_pung(p,the_game->tile) ) {
@@ -645,7 +691,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
     player_pungs(&pc,the_game->tile);
     decide_discard(&pc,&val,&curstrat);
     if ( debugeval ) {
-      printf("Hand after pung  %.3f\n",val);
+      fprintf(debugf,"Hand after pung  %.3f\n",val);
     }
     if ( val > bestval ) {
       m.type = PMsgPung;
@@ -654,7 +700,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
     }
     else 
       if ( debugeval ) {
-	printf("Chose not to pung\n");
+	fprintf(debugf,"Chose not to pung\n");
       }
     }
   if ( (canmj || our_seat == (the_game->player+1)%NUM_SEATS)
@@ -667,7 +713,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
     if ( player_chows(&pc,t,Lower) ) {
       decide_discard(&pc,&val,&curstrat);
       if ( debugeval ) {
-	printf("Hand after lower chow: %.3f\n",val);
+	fprintf(debugf,"Hand after lower chow: %.3f\n",val);
       }
       chowposs = 1;
       if ( val > bestval ) {
@@ -679,7 +725,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
     if ( player_chows(&pc,t,Middle) ) {
       decide_discard(&pc,&val,&curstrat);
       if ( debugeval ) {
-	printf("Hand after middle chow: %.3f\n",val);
+	fprintf(debugf,"Hand after middle chow: %.3f\n",val);
       }
       chowposs = 1;
       if ( val > bestval ) {
@@ -692,7 +738,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
       chowposs = 1;
       decide_discard(&pc,&val,&curstrat);
       if ( debugeval ) {
-	printf("Hand after upper chow: %.3f\n",val);
+	fprintf(debugf,"Hand after upper chow: %.3f\n",val);
       }
       if ( val > bestval ) {
 	bestval = val;
@@ -707,7 +753,7 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
     } 
     else
       if ( debugeval ) {
-	if ( chowposs ) fprintf(stdout,"chose not to chow\n");
+	if ( chowposs ) fprintf(debugf,"chose not to chow\n");
       }
   }
   /* mah jong */
@@ -730,17 +776,17 @@ static void check_discard(PlayerP p, strategy *strat,int closedkong) {
     }
 #endif
     if ( m.type != PMsgNoClaim ) {
-      if ( debugeval && strat->hiddenness > 0.0 ) printf("claiming mahjong on hidden strategy\n");
+      if ( debugeval && strat->hiddenness > 0.0 ) fprintf(debugf,"claiming mahjong on hidden strategy\n");
       m.type = PMsgMahJong;
       m.mahjong.discard = the_game->serial;
     } else {
       if ( debugeval ) {
-	printf("CHOSE NOT TO MAHJONG\n");
+	fprintf(debugf,"CHOSE NOT TO MAHJONG\n");
       }
     }
   }
   if ( debugeval ) {
-    printf("Result: %s",encode_pmsg((PMsgMsg *)&m));
+    fprintf(debugf,"Result: %s",encode_pmsg((PMsgMsg *)&m));
   }
   send_packet(&m);
   return;
@@ -882,8 +928,8 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
 		   double *breadth) {
   Tile copy[MAX_CONCEALED+1],copy2[MAX_CONCEALED+1];
   int i;
-  double mval = -1000, pval = -1000, val = -1000,
-    pvalr = -1000, mvalr = -1000, valr = -1000;
+  double mval = 0, pval = 0, val = 0,
+    pvalr = 0, mvalr = 0, valr = 0;
   int ppr, mpr; int spr;
   char prefix[200];
   static char totbuf[200];
@@ -902,14 +948,14 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       for ( ; i < MAX_CONCEALED+1 ; i++ ) copy[i] = HiddenTile;
       val = eval(copy,strat,stratpoints,0,ninc,npr,breadth);
       mval += val ;
-      if ( debugeval ) printf("Evaling suit %d returned v=%.2f, b=%.2f\n",
+      if ( debugeval ) fprintf(debugf,"Evaling suit %d returned v=%.2f, b=%.2f\n",
 			      suit_of(copy[0]),val,*breadth);
       tcopy(copy,copy2);
       for ( i= 0; suit_of(copy[i]) == suit_of(copy2[0]) ; i++ )
 	copy[i] = HiddenTile;
       tsort(copy);
     }
-    if ( debugeval ) printf("Level 0 returned %.2f\n",mval);
+    if ( debugeval ) fprintf(debugf,"Level 0 returned %.2f\n",mval);
     return mval;
   }
 
@@ -928,7 +974,7 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
 
   if ( tp[0] == HiddenTile ) {
     if ( debugeval > 1 ) {
-      printf("\n"); 
+      fprintf(debugf,"\n"); 
       if (reclevel) sprintf(&totbuf[11*(reclevel-1)+4],"\n");
     }
     /* if there is one "incomplete" set and one pair, we have a complete
@@ -955,16 +1001,16 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       */
       val += stratpoints[pungbase];
       /* if we're trying for a no score hand, scoring pairs are bad */
-      if ( is_doubler(tp[0]) ) val += (-strat->chowness)  * 6.0;
+      if ( is_doubler(tp[0]) ) val += (-strat->chowness)  * magic2[0];
       if ( is_major(tp[0]) ) { 
-	val += 2.0;
+	val += magic2[1];
       } else {
-	val -= 25.0*strat->majorness;
+	val -= magic2[2]*strat->majorness;
       }
       if ( is_suit(tp[0]) && suit_of(tp[0]) != strat->suit ) 
 	val *= stratpoints[suitfactor];
       if ( debugeval > 1 ) {
-	printf("%sPu%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
+	fprintf(debugf,"%sPu%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
       }
       /* remove the tiles and recurse */
       tcopy(copy,tp);
@@ -991,14 +1037,14 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       /* doublers are bad for noscore */
       if ( is_doubler(tp[0]) ) val += 2 - (strat->chowness)*4.0;
       if ( is_major(tp[0]) ) {
-	val += 1.0;
+	val += magic2[3];
       } else {
-	val -= 12.0*strat->majorness;
+	val -= magic2[4]*strat->majorness;
       }
       if ( is_suit(tp[0]) && suit_of(tp[0]) != strat->suit ) 
 	val *= stratpoints[suitfactor];
       if ( debugeval > 1 ) {
-	printf("%sPr%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
+	fprintf(debugf,"%sPr%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
       }
       /* remove the tiles and recurse */
       tcopy(copy,tp);
@@ -1035,8 +1081,8 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       if ( is_suit(tp[0]) && suit_of(tp[0]) != strat->suit ) 
 	val *= stratpoints[suitfactor];
      if ( debugeval > 1 ) {
-       if ( notfirst ) printf("%s%s",prefix,&totbuf[11*reclevel]);
-       printf("%sCh%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
+       if ( notfirst ) fprintf(debugf,"%s%s",prefix,&totbuf[11*reclevel]);
+       fprintf(debugf,"%sCh%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
      }
       /* remove the tiles and recurse */
       tcopy(copy,tp);
@@ -1071,8 +1117,8 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       if ( is_suit(tp[0]) && suit_of(tp[0]) != strat->suit ) 
 	val *= stratpoints[suitfactor];
       if ( debugeval > 1 ) {
-	if ( notfirst ) printf("%s%s",prefix,&totbuf[11*reclevel]);
-	printf("%sIn%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
+	if ( notfirst ) fprintf(debugf,"%s%s",prefix,&totbuf[11*reclevel]);
+	fprintf(debugf,"%sIn%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
       }
       /* remove the tiles and recurse */
       tcopy(copy,tp);
@@ -1096,8 +1142,8 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       if ( is_suit(tp[0]) && suit_of(tp[0]) != strat->suit ) 
 	val *= stratpoints[suitfactor];
       if ( debugeval > 1 ) {
-	if ( notfirst ) printf("%s%s",prefix,&totbuf[11*reclevel]);
-	printf("%sOu%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
+	if ( notfirst ) fprintf(debugf,"%s%s",prefix,&totbuf[11*reclevel]);
+	fprintf(debugf,"%sOu%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
       }
       /* remove the tiles and recurse */
       tcopy(copy,tp);
@@ -1119,28 +1165,28 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
   val += stratpoints[sglbase]
     * (tilesleft[tp[0]]
        + ( is_suit(tp[0])
-	  ? (0.5*strat->chowness*(tilesleft[tp[0]-1]+ tilesleft[tp[0]+1])) : 0))
-    * (3.0-strat->chowness);
+	  ? (magic2[5]*strat->chowness*(tilesleft[tp[0]-1]+ tilesleft[tp[0]+1])) : 0))
+    * (magic2[6]-strat->chowness);
   if ( is_doubler(tp[0]) ) {
-    if (tilesleft[tp[0]] == 0) val -= 5.0; /* completely useless */
+    if (tilesleft[tp[0]] == 0) val -= magic2[7]; /* completely useless */
     else {
-      if ( (0.5-strat->chowness) > 0 ) 
-	val += (tilesleft[tp[0]])*stratpoints[sglbase]*(0.5-strat->chowness);
+      if ( (magic2[8]-strat->chowness) > 0 ) 
+	val += (tilesleft[tp[0]])*stratpoints[sglbase]*(magic2[9]-strat->chowness);
       else
-	val += (3-tilesleft[tp[0]])*stratpoints[sglbase]*(0.5-strat->chowness);
+	val += (3-tilesleft[tp[0]])*stratpoints[sglbase]*(magic2[10]-strat->chowness);
     }
   }
   if ( is_suit(tp[0]) && suit_of(tp[0]) != strat->suit ) {
     /* val could be negative, in which case we don't want to shrink it.
        So just substract a constant */
-    val -= 0.5*strat->suitness*stratpoints[sglbase];
+    val -= magic2[11]*strat->suitness*stratpoints[sglbase];
   }
   if ( ! is_major(tp[0]) ) {
-    val -= 0.5*strat->majorness*stratpoints[sglbase];
+    val -= magic2[12]*strat->majorness*stratpoints[sglbase];
   }
   if ( debugeval > 1 ) {
-    if ( notfirst ) printf("%s%s",prefix,&totbuf[11*reclevel]);
-    printf("%sSi%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
+    if ( notfirst ) fprintf(debugf,"%s%s",prefix,&totbuf[11*reclevel]);
+    fprintf(debugf,"%sSi%s:%4.1f+ ",notfirst++?prefix:"",tile_code(tp[0]),val);
   }
   tcopy(copy,tp);
   copy[0] = HiddenTile;
@@ -1173,7 +1219,7 @@ static double eval(Tile *tp,strategy *strat,double *stratpoints,
       sprintf(tb,"(%4.1f) ",mval);
       strncpy(&totbuf[11*(reclevel-1)+4],tb,7);
     }
-    else printf(totbuf);
+    else fprintf(debugf,totbuf);
   }
   return mval;
 }
@@ -1189,40 +1235,40 @@ static double evalhand(PlayerP p,strategy *strat) {
 
   if ( debugeval ) {
     char buf[100];
-    printf("eval with strat params c=%.1f, h=%.1f, m=%.1f, s=%.1f (%d)\n",
+    fprintf(debugf,"eval with strat params c=%.1f, h=%.1f, m=%.1f, s=%.1f (%d)\n",
 	   strat->chowness,strat->hiddenness,strat->majorness,
 	   strat->suitness,strat->suit);
     player_print_tiles(buf,p,0);
-    printf("Hand: %s\n",buf);
+    fprintf(debugf,"Hand: %s\n",buf);
   }
   /* calculate old style strategy values from new ones */
   stratpoints[pungbase] = 
-    12.0 - ( strat->chowness > 0 ? strat->chowness * 12.0 
-	     : strat->chowness * 2.0 );
-  stratpoints[pungbase] *= (1.0 + 0.15 * strat->hiddenness);
-  stratpoints[pairbase] = 1.0*(4.0 + (2.0*fabs(strat->chowness)));
+    magic[0] - ( strat->chowness > 0 ? strat->chowness * magic[0] 
+	     : strat->chowness * magic[1] );
+  stratpoints[pungbase] *= (magic[2] + magic[3] * strat->hiddenness);
+  stratpoints[pairbase] = magic[4]*(magic[5] + (magic[6]*fabs(strat->chowness)));
   stratpoints[chowbase] = 
-    12.0 + ( strat->chowness > 0 ? strat->chowness * 4.0
-	     : strat->chowness * 12.0 );
-  stratpoints[chowbase] *= (1.0 + 0.15 * strat->hiddenness)
+    magic[0] + ( strat->chowness > 0 ? strat->chowness * magic[7]
+	     : strat->chowness * magic[0] );
+  stratpoints[chowbase] *= (magic[8] + magic[9] * strat->hiddenness)
     * (1.0 - strat->majorness);
-  stratpoints[seqbase] = 0*(1.0 + ( strat->chowness < 0 ? strat->chowness : 0))
+  stratpoints[seqbase] = magic[10]*(magic[11] + ( strat->chowness < 0 ? strat->chowness : 0))
     * (1.0 - strat->majorness);
-  stratpoints[sglbase] = 0.025;
-  stratpoints[partpung] = 0.4*(0.34)*(1-0.4*strat->hiddenness);
-  stratpoints[partchow] = 0.4*(1.0/12.0)*(1-0.3*strat->hiddenness) * (1.0 - strat->majorness);
-  stratpoints[exposedpungpenalty] = 2.0 + strat->hiddenness * stratpoints[pungbase]
-    + ( (strat->chowness > 0) ? 25.0 * strat->chowness : 0.0 );
-  stratpoints[exposedchowpenalty] = 2.0 + strat->hiddenness * stratpoints[chowbase]
-    + ( (strat->chowness < 0) ? -25.0 * strat->chowness : 0.0 )
-    + 25.0 * strat->majorness;
-  stratpoints[mjbonus] = 60.0;
-  stratpoints[kongbonus] = 3.0;
+  stratpoints[sglbase] = magic[12];
+  stratpoints[partpung] = magic[13]*(magic[14])*(1-magic[15]*strat->hiddenness);
+  stratpoints[partchow] = magic[16]*(1-magic[17]*strat->hiddenness) * (1.0 - strat->majorness);
+  stratpoints[exposedpungpenalty] = magic[18] + strat->hiddenness * stratpoints[pungbase]
+    + ( (strat->chowness > 0) ? magic[19] * strat->chowness : 0.0 );
+  stratpoints[exposedchowpenalty] = magic[20] + strat->hiddenness * stratpoints[chowbase]
+    + ( (strat->chowness < 0) ? -1*magic[21] * strat->chowness : 0.0 )
+    + magic[21] * strat->majorness;
+  stratpoints[mjbonus] = magic[22];
+  stratpoints[kongbonus] = magic[23];
   stratpoints[suitfactor] = 
-    ((strat->suitness >= 1.0) ? 0.01 : (1.0 - 0.99*strat->suitness));
-  stratpoints[weight] = 1.0 + 
-    (strat->suitness >= 1.0 ? 0.2*(strat->suitness-1.0) : 0.2*strat->suitness)
-    + 0.25*strat->majorness;
+    ((strat->suitness >= 1.0) ? 0.01 : (1.0 - magic[24]*strat->suitness));
+  stratpoints[weight] = magic[25] + 
+    (strat->suitness >= 1.0 ? magic[26]*(strat->suitness-1.0) : magic[26]*strat->suitness)
+    + magic[27]*strat->majorness;
 
   for (i=0; i<p->num_concealed; i++)
     tcopy[i] = p->concealed[i];
@@ -1273,7 +1319,7 @@ static double evalhand(PlayerP p,strategy *strat) {
 	 && suit_of(p->tilesets[i].tile) != strat->suit )
       sval -= 10.0*strat->suitness;
     if ( debugeval > 1 ) {
-      printf("Set %s: %.1f\n",player_print_TileSetType(p->tilesets[i].type),
+      fprintf(debugf,"Set %s: %.1f\n",player_print_TileSetType(p->tilesets[i].type),
 	     sval);
     }
     val += sval;
@@ -1282,7 +1328,7 @@ static double evalhand(PlayerP p,strategy *strat) {
   val += eval(tcopy,strat,stratpoints,-1,&ninc,&npr,&breadth);
   /* add the pairbase if we have at least one pair */
   if ( npr > 0 ) {
-    if ( debugeval > 1 ) printf("+pairbase %.1f\n",stratpoints[pairbase]);
+    if ( debugeval > 1 ) fprintf(debugf,"+pairbase %.1f\n",stratpoints[pairbase]);
     val += stratpoints[pairbase];
   }
   val *= stratpoints[weight];
@@ -1292,7 +1338,7 @@ static double evalhand(PlayerP p,strategy *strat) {
   r = 4*r*r;
   return val+r;
 #else
-  if ( debugeval ) printf("Value %.2f\n",val);
+  if ( debugeval ) fprintf(debugf,"Value %.2f\n",val);
   return val;
 #endif
 }
@@ -1337,7 +1383,7 @@ static Tile decide_discard(PlayerP p, double *score, strategy *strat) {
   
   if ( debugeval ) {
     player_print_tiles(buf,p,0);
-    printf("Hand: %s\n",buf);
+    fprintf(debugf,"Hand: %s\n",buf);
   }
   best = 0;
   tsort(tilesa); /* no point in looking at same tile twice */
@@ -1345,7 +1391,7 @@ static Tile decide_discard(PlayerP p, double *score, strategy *strat) {
     Player cp;
     if ( i && tilesa[i] == tilesa[i-1] ) continue;
     if ( debugeval ) {
-      printf("Trying %s: \n",tile_code(tilesa[i]));
+      fprintf(debugf,"Trying %s: \n",tile_code(tilesa[i]));
     }
     copy_player(&cp,p);
     player_discards(&cp,tilesa[i]);
@@ -1363,11 +1409,11 @@ static Tile decide_discard(PlayerP p, double *score, strategy *strat) {
     /* Best tile to discard leaves highest residual score */
     if ( values[i] > values[best] ) best = i;
     if ( debugeval ) {
-	printf("Tile %s has value %.1f\n",tile_code(tilesa[i]),values[i]);
+	fprintf(debugf,"Tile %s has value %.1f\n",tile_code(tilesa[i]),values[i]);
     }
   }
   if ( debugeval ) {
-    printf("Discarding %s\n",tile_code(tilesa[best]));
+    fprintf(debugf,"Discarding %s\n",tile_code(tilesa[best]));
   }
   if ( score ) *score = values[best];
   return tilesa[best];
@@ -1459,17 +1505,17 @@ static void maybe_switch_strategy(strategy *strat) {
 #endif
   if ( dofast ) {
     /* strat = Fast; */
-    if ( debugeval ) { printf("Using fast\n"); }
+    if ( debugeval ) { fprintf(debugf,"Using fast\n"); }
   } else {
     if ( bestval >= oval + hysteresis ) {
       *strat = beststrat;
       if ( debugeval ) {
 	static char buf[128];
-	printf("Switching to strat c=%.1f h=%.1f s=%.1f (%d) m=%.1f\n",
+	fprintf(debugf,"Switching to strat c=%.1f h=%.1f s=%.1f (%d) m=%.1f\n",
 	       strat->chowness,strat->hiddenness,
 	       strat->suitness,strat->suit,strat->majorness);
 	  player_print_tiles(buf,p,0);
-	  printf(" with hand %s\n",buf);
+	  fprintf(debugf," with hand %s\n",buf);
       }
     }
   }
