@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/scoring.c,v 11.8 2003/04/13 18:49:51 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/scoring.c,v 11.9 2005/03/22 21:11:59 jcb Exp $
  * scoring.c 
  * This file contains the routines to handle the scoring
  * of hands.
@@ -15,7 +15,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/scoring.c,v 11.8 2003/04/13 18:49:51 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/scoring.c,v 11.9 2005/03/22 21:11:59 jcb Exp $";
 
 #include "tiles.h"
 #include "player.h"
@@ -161,11 +161,11 @@ Score score_of_hand(Game *g, seats s) {
     clims = ((score) / 1000000); \
     if ( (!clims || nolimit) && pts ) { \
       tot.value += pts; sprintf(tbuf,"%-12s %2d (%s)\n",ltext,pts,rtext); \
-      strcat(buf,tbuf); \
+      strmcat(buf,tbuf,sizeof(buf)-strlen(buf)); \
     } \
     if ( (!clims || nolimit) && dbls ) { \
       doubles += dbls; sprintf(tbuf,"%-12s  %1d dbl (%s)\n",ltext,dbls,rtext); \
-      strcat(dblsbuf,tbuf); \
+      strmcat(dblsbuf,tbuf,sizeof(dblsbuf)-strlen(dblsbuf)); \
     } \
     if ( clims && !(nolimit && (pts || dbls)) ) { \
       if ( clims > centilims ) centilims = clims; \
@@ -174,7 +174,7 @@ Score score_of_hand(Game *g, seats s) {
       } else { \
         sprintf(tbuf,"%4g Limit %4d (%s)\n",clims*1.0/100,clims*limit/100,rtext); \
       } \
-      strcat(limsbuf,tbuf); \
+      strmcat(limsbuf,tbuf,sizeof(limsbuf)-strlen(limsbuf)); \
     } \
   }
   /* and a couple of defines for hard-coded scores */
@@ -729,14 +729,14 @@ Score score_of_hand(Game *g, seats s) {
 
   /* display the total points so far */
   sprintf(tbuf,"%-11s %3d\n","total pts",tot.value);
-  strcat(buf,tbuf);
+  strmcat(buf,tbuf,sizeof(buf)-strlen(buf));
 
   /* add the double descriptions */
-  strcat(buf,dblsbuf);
+  strmcat(buf,dblsbuf,sizeof(buf)-strlen(buf));
 
   /* calculate the doubles */
   sprintf(tbuf,"%-12s %2d\n","total dbls",doubles);
-  strcat(buf,tbuf);
+  strmcat(buf,tbuf,sizeof(buf)-strlen(buf));
   /* to avoid arithmetic overflow when people set crazy options,
      we'll put a hard limit of 1E8 in */
   while ( doubles-- > 0 ) {
@@ -745,12 +745,12 @@ Score score_of_hand(Game *g, seats s) {
   }
   if ( ! nolimit && tot.value > limit ) {
     sprintf(tbuf,"%-9s %5d (over limit)\n","score:",tot.value);
-    strcat(buf,tbuf);
+    strmcat(buf,tbuf,sizeof(buf)-strlen(buf));
     tot.value = limit;
   }
 
   /* and the limit descriptions */
-  strcat(buf,limsbuf);
+  strmcat(buf,limsbuf,sizeof(buf)-strlen(buf));
   /* calculate the limit */
 
   if ( centilims ) {
@@ -763,7 +763,7 @@ Score score_of_hand(Game *g, seats s) {
     }
   }
   sprintf(tbuf,"%-9s %5d","SCORE:",tot.value);
-  strcat(buf,tbuf);
+  strmcat(buf,tbuf,sizeof(buf)-strlen(buf));
 
   /* just return what we've got, to see if it's working */
   return tot;
