@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/sysdep.c,v 11.31 2006/01/22 21:02:37 jcb Exp $
+/* $Header: /home/jcb/newmj/RCS/sysdep.c,v 11.32 2006/01/24 22:06:59 jcb Exp $
  * sysdep.c
  * By intention, this file contains all functions that might need
  * fiddling with to cope with different variants of Unix.
@@ -21,7 +21,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.31 2006/01/22 21:02:37 jcb Exp $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.32 2006/01/24 22:06:59 jcb Exp $";
 
 #include "sysdep.h"
 #include <string.h>
@@ -429,7 +429,12 @@ static SOCKET _connect_to_host(int plain, const char *address) {
       data = 300;
       n = setsockopt(fd, SOL_TCP, TCP_KEEPIDLE, &data, sizeof(int));
       if ( n < 0 ) {
-	perror("connect_to_host: setsockopt keepidle failed");
+	/* of course, this is not supported for unix sockets. We could
+	   have checked the address family, but it's probably simpler
+	   just to ignore errors */
+	if ( errno != ENOTSUP ) {
+	  perror("connect_to_host: setsockopt keepidle failed");
+	}
       }
 #endif
     }
